/**
 * expandclick 图标点击事件触发区域扩大器
 * @params {element} el是目标元素
 * 注意：
 * 1.加指令的元素必须是定位元素!
 * 2.传递参数的时候，因为考虑到居中问题，建议传递一样的参数值，确保可以居中。
 * 3.绑定指令的元素 不能设置overflow属性 否则会失去效果。
 *
 * @example
 * <div><i v-expandclick:10*20*30*40></i></div>
 * v-expandclick:10  一个值代表上下左右
 * v-expandclick:10*20   两个值对应【上下】【左右】
 * v-expandclick:10*20*30  三个值对应 【上】【左右】【下】
 * v-expandclick:10*20*30*40  四个值对应 【上】【右】【下】【左】
 * v-expandclick     不填参数则默认为10px间距
 * 总结一下，规则就跟css的margin、padding的值填写规则一样。
 */
import isStringEmpty from "../../utils/isStringEmpty";

const expandclick = {
    // install方法
    install(Vue, option) {
        //expandSize 默认的扩展热区半径(单位px)
        let expandSize = 10
        if (option && option.expandSize && typeof option.expandSize === 'number') {
            expandSize = option.expandSize
        }
        Vue.directive('expandclick', {
            inserted(el, binding) {
                expandclick.inserted(el, binding, expandSize)
            }
        })
    },
    /**
     * 自定义指令的钩子函数 inserted 被绑定元素插入父节点时调用（父节点存在即可调用，不必存在于 document 中）
     * @param  {element} $el - 目标dom元素
     * @param  {object} binding - 指令对象
     */
    inserted($el, binding, expandSize) {
        try {
            const pos = window.getComputedStyle($el)
            if (!pos.position || pos.position === 'static') {
                $el.style.position = 'relative'
            }

            // 原生创建一个span 作为插入的虚拟节点
            const ficSpan = document.createElement('span')
            //创建之后插入到$el实例中
            $el.appendChild(ficSpan)

            let argList = []
            if (isStringEmpty(binding.arg)) {
                // 默认为10px间距
                argList = [expandSize]
            } else {
                argList = binding.arg.split('*')
            }

            const offset = {
                // 用于调整至中心位置
                top: 0,
                left: 0,
            }
            // 测试用例背景色
            ficSpan.style.background = 'transparent'
            const posWidth = parseInt(pos.width)
            const posHeight = parseInt(pos.height)
            switch (argList.length) {
                case 1: {
                    // 上下左右四个方向都是同样的值
                    const length = parseInt(argList[0]) * 2
                    let verticalSize = (posWidth + length) / 2
                    let horizontalSize = (posHeight + length) / 2
                    ficSpan.style.paddingLeft = `${verticalSize}px`
                    ficSpan.style.paddingRight = `${verticalSize}px`
                    ficSpan.style.paddingTop = `${horizontalSize}px`
                    ficSpan.style.paddingBottom = `${horizontalSize}px`
                    offset.top = offset.left = length
                    break
                }
                case 2: {
                    const height = parseInt(argList[0]) * 2
                    const width = parseInt(argList[1]) * 2
                    let verticalSize = (posWidth + width) / 2
                    let horizontalSize = (posHeight + height) / 2
                    ficSpan.style.paddingLeft = `${verticalSize}px`
                    ficSpan.style.paddingRight = `${verticalSize}px`
                    ficSpan.style.paddingTop = `${horizontalSize}px`
                    ficSpan.style.paddingBottom = `${horizontalSize}px`
                    offset.left = width
                    offset.top = height
                    break
                }
                case 3: {
                    const top = parseInt(argList[0])
                    const bottom = parseInt(argList[2])
                    const width = parseInt(argList[1])
                    ficSpan.style.paddingLeft = `${(posWidth + (width * 2)) / 2}px`
                    ficSpan.style.paddingRight = `${(posWidth + (width * 2)) / 2}px`
                    ficSpan.style.paddingTop = `${(posHeight + (top * 2)) / 2}px`
                    ficSpan.style.paddingBottom = `${(posHeight + (bottom * 2)) / 2}px`
                    offset.left = width * 2
                    offset.top = top * 2
                    break
                }
                case 4: {
                    const top = parseInt(argList[0])
                    const right = parseInt(argList[1])
                    const bottom = parseInt(argList[2])
                    const left = parseInt(argList[3])
                    ficSpan.style.paddingLeft = `${(posWidth + (left * 2)) / 2}px`
                    ficSpan.style.paddingRight = `${(posWidth + (right * 2)) / 2}px`
                    ficSpan.style.paddingTop = `${(posHeight + (top * 2)) / 2}px`
                    ficSpan.style.paddingBottom = `${(posHeight + (bottom * 2)) / 2}px`
                    offset.left = left * 2
                    offset.top = top * 2
                    break
                }
            }
            // 绝对定位设置在父元素的中心
            ficSpan.style.position = 'absolute'
            ficSpan.style.top = '0'
            ficSpan.style.left = '0'
            ficSpan.style.marginTop = `-${offset.top / 2}px`
            ficSpan.style.marginLeft = `-${offset.left / 2}px`
        } catch (e) {
            console.error('[easy-project-vue warn]:', e)
        }

    },
}

export default expandclick
